@@ -5,6 +5,8 @@ import sys
 import unittest
 import xml
 import pickle
+import threading
+import json
 
 import pytest
 
@@ -19,6 +21,12 @@ if sys.version_info < (3, 2):
 
 data_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
 
+def _get_library_version():
+    lib, handle, lib_version_str, lib_version = MediaInfo._get_library()
+    lib.MediaInfo_Close(handle)
+    lib.MediaInfo_Delete(handle)
+    return lib_version_str, lib_version
+
 class MediaInfoTest(unittest.TestCase):
     def setUp(self):
         with open(os.path.join(data_dir, 'sample.xml'), 'r') as f:
@@ -96,7 +104,7 @@ class MediaInfoUnicodeFileNameTest(unittest.TestCase):
 
 class MediaInfoURLTest(unittest.TestCase):
     def setUp(self):
-        self.mi = MediaInfo.parse("https://github.com/sbraz/pymediainfo/blob/master/tests/data/sample.mkv?raw=true")
+        self.mi = MediaInfo.parse("https://github.com/sbraz/pymediainfo/raw/master/tests/data/sample.mkv")
     def test_parse_url(self):
         self.assertEqual(len(self.mi.tracks), 2)
 
@@ -112,6 +120,33 @@ class MediaInfoPathlibTest(unittest.TestCase):
         path = self.pathlib.Path(data_dir) / "this file does not exist"
         self.assertRaises(FileNotFoundError, MediaInfo.parse, path)
 
+class MediaInfoFilenameTypesTest(unittest.TestCase):
+    def test_parse_filename_str(self):
+        path = os.path.join(data_dir, "test.txt")
+        filename, is_url = MediaInfo._parse_filename(path)
+        # Windows paths are URLs
+        if not os_is_nt:
+            self.assertFalse(is_url)
+        self.assertEqual(filename, path)
+    def test_parse_filename_pathlib(self):
+        pathlib = pytest.importorskip("pathlib")
+        path = pathlib.Path(data_dir, "test.txt")
+        filename, is_url = MediaInfo._parse_filename(path)
+        self.assertFalse(is_url)
+        self.assertEqual(filename, os.path.join(data_dir, "test.txt"))
+    @pytest.mark.skipif(sys.version_info < (3, 6), reason="os.PathLike requires Python 3.6")
+    def test_parse_filename_pathlike(self):
+        class PathLikeObject(os.PathLike):
+            def __fspath__(self):
+                return os.path.join(data_dir, "test.txt")
+        path = PathLikeObject()
+        filename, is_url = MediaInfo._parse_filename(path)
+        self.assertFalse(is_url)
+        self.assertEqual(filename, os.path.join(data_dir, "test.txt"))
+    def test_parse_filename_url(self):
+        filename, is_url = MediaInfo._parse_filename("https://localhost")
+        self.assertTrue(is_url)
+
 class MediaInfoTestParseNonExistentFile(unittest.TestCase):
     @pytest.mark.skipif(os_is_nt, reason="Windows paths are URLs")
     def test_parse_non_existent_path(self):
@@ -134,9 +169,9 @@ class MediaInfoCoverDataTest(unittest.TestCase):
                 "AAAQCEeRdzAAAADUlEQVR4nGP4x8DwHwAE/AH+QSRCQgAAAABJRU5ErkJggg=="
         )
     def test_parse_no_cover_data(self):
-        _, lib_version_str, lib_version = MediaInfo._get_library()
+        lib_version_str, lib_version = _get_library_version()
         if lib_version < (18, 3):
-            pytest.skip("Cover_Data option not supported by this library version "
+            pytest.skip("The Cover_Data option is not supported by this library version "
                 "(v{} detected, v18.03 required)".format(lib_version_str)
             )
         self.assertEqual(self.no_cover_mi.tracks[0].cover_data, None)
@@ -194,9 +229,9 @@ class MediaInfoLegacyStreamDisplayTest(unittest.TestCase):
 
 class MediaInfoOptionsTest(unittest.TestCase):
     def setUp(self):
-        _, lib_version_str, lib_version = MediaInfo._get_library()
+        lib_version_str, lib_version = _get_library_version()
         if lib_version < (19, 9):
-            pytest.skip("Reset option not supported by this library version "
+            pytest.skip("The Reset option is not supported by this library version "
                 "(v{} detected, v19.09 required)".format(lib_version_str)
             )
         self.raw_language_mi = MediaInfo.parse(
@@ -212,3 +247,56 @@ class MediaInfoOptionsTest(unittest.TestCase):
     def test_mediainfo_options(self):
         self.assertEqual(self.normal_mi.tracks[1].other_language[0], "English")
         self.assertEqual(self.raw_language_mi.tracks[1].language, "en")
+
+# Unittests can't be parametrized
+# https://github.com/pytest-dev/pytest/issues/541
+@pytest.mark.parametrize("test_file", ["sample.mkv", "sample.mp4", "sample_with_cover.mp3"])
+def test_thread_safety(test_file):
+    lib_version_str, lib_version = _get_library_version()
+    if lib_version < (20, 3):
+        pytest.skip("This version of the library is not thread-safe "
+            "(v{} detected, v20.03 required)".format(lib_version_str)
+        )
+    expected_result = MediaInfo.parse(os.path.join(data_dir, test_file))
+    results = []
+    lock = threading.Lock()
+    def target():
+        try:
+            result = MediaInfo.parse(os.path.join(data_dir, test_file))
+            with lock:
+                results.append(result)
+        except:
+            pass
+    threads = []
+    thread_count = 100
+    for i in range(thread_count):
+        t = threading.Thread(target=target)
+        t.start()
+        threads.append(t)
+    for t in threads:
+        t.join()
+    # Each thread should have produced a result
+    assert len(results) == thread_count
+    maxDiff = None
+    for r in results:
+        # Test dicts first because they will show a diff
+        # in case they don't match
+        assert r.to_data() == expected_result.to_data()
+        assert r == expected_result
+
+class MediaInfoOutputTest(unittest.TestCase):
+    def test_text_output(self):
+        mi = MediaInfo.parse(os.path.join(data_dir, "sample.mp4"), output="")
+        self.assertRegex(mi, r"Stream size\s+: 373836\b")
+    def test_json_output(self):
+        lib_version_str, lib_version = _get_library_version()
+        if lib_version < (18, 3):
+            pytest.skip("This version of the library does not support JSON output "
+                "(v{} detected, v18.03 required)".format(lib_version_str)
+            )
+        mi = MediaInfo.parse(os.path.join(data_dir, "sample.mp4"), output="JSON")
+        parsed = json.loads(mi)
+        self.assertEqual(parsed["media"]["track"][0]["FileSize"], "404567")
+    def test_parameter_output(self):
+        mi = MediaInfo.parse(os.path.join(data_dir, "sample.mp4"), output="General;%FileSize%")
+        self.assertEqual(mi, "404567")
